;LMX2541CTL.ASM     G4JNt  Dec 2011
;  Based on basicSPI with added register storage

;Simple ASCII/Hex to SPI commands

;  Serial Command  protocol     
;	Rxxxxxxxx	32 bit ASCII data, terminated with [cr]  (Must be an EVEN number of x digits)
;       TEST	Send chip data continuously for SPI testing

;Typo corrected, SerIn on line 664 only revealed with RS232POL = 0.  This version tested OK with FTDI232 interface @ 5V
; _DUAL version uses link on GP3 to select one of two pages of EE data.  For programming or settign synth.


    PROCESSOR 12F629
    INCLUDE <P12F629.INC>
    errorlevel 1



;   ===============================================================

    #define     SerIn       GPIO, 0     ;Also PGD,  Port change enabled  Pin 7
    #define     SerOut      GPIO, 1     ;Also PGC			 Pin 6
    #define     Data   	    GPIO, 5     ;Pin 2
    #define     Clk   	    GPIO, 4     ;Pin 3
    #define     LE	    GPIO, 2     ;Pin 5
    #define     PAGELINK    GPIO, 3     ;Pin 4 (no active pull up)

	#define	ContMode	Flags, 0
	#define	Page2		Flags, 7

    #define     LED         GPIO, 1     ;LED on SerOut for debugging

 RS232POL   =   0         ; 1 = RS232 levels,  0 = Logic/FTDI Interface
 BAUDCONST  =   d'14'      ;6.N + 18  =  1000000/Baud   Half baud delay, 9600 baud

  __config  0x3FC4      ;INTOSC, I/O on GP4/5
  ;CPD Off, CP Off, Boden On, MCLRE = I/O, PWRTE on, WDT Off,

  __idlocs  0x806C



  org	0x2100


ConfigData		;32 bit words sent in sequence
  ;  Fout  564MHz  Resolution  10Hz
  ;  R =  1      D =  166667      N =  338      F =  66667
   de  0x00, 0x00, 0x00, 0x17    ;  Reg7   
   de  0x00, 0x00, 0x00, 0xCD    ;  Reg13   
   de  0x00, 0x00, 0x00, 0x1C    ;  Reg12   
   de  0x28, 0x00, 0x14, 0x09    ;  Reg9   
   de  0x01, 0x11, 0xCE, 0x58    ;  Reg8   
   de  0x00, 0x1F, 0x30, 0xE6    ;  Reg6   
   de  0xA0, 0x04, 0x00, 0x05    ;  Reg5   
   de  0xFF, 0x48, 0x60, 0xA4    ;  Reg4   
   de  0x00, 0xA8, 0x7F, 0x03    ;  Reg3   
   de  0x04, 0x28, 0xB0, 0xB2    ;  Reg2   
   de  0x00, 0x40, 0x00, 0x11    ;  Reg1   
   de  0x04, 0x6B, 0x15, 0x20    ;  Reg0           
   
EndConfigData

 org	0x2140

ConfigData2nd		;32 bit words sent in sequence
  ;  Fout  576MHz  Resolution  10Hz
  ;  R =  1      D =  166667      N =  345      F =  100000
   de  0x00, 0x00, 0x00, 0x17    ;  Reg7   
   de  0x00, 0x00, 0x00, 0xCD    ;  Reg13   
   de  0x00, 0x00, 0x00, 0x1C    ;  Reg12   
   de  0x28, 0x00, 0x14, 0x09    ;  Reg9   
   de  0x01, 0x11, 0xCE, 0x58    ;  Reg8   
   de  0x00, 0x1F, 0x30, 0xE6    ;  Reg6   
   de  0xA0, 0x04, 0x00, 0x05    ;  Reg5   
   de  0xFF, 0x48, 0x60, 0xA4    ;  Reg4   
   de  0x00, 0xA8, 0x7F, 0x03    ;  Reg3   
   de  0x04, 0x28, 0xB0, 0xB2    ;  Reg2   
   de  0x00, 0x40, 0x00, 0x11    ;  Reg1   
   de  0x86, 0xA0, 0x15, 0x90    ;  Reg0            
     

EndConfigData2nd



;==================	 
  org   0
    nop
    clrw
    movwf   INTCON          ;disable interrupts
    goto    StartUp         ;jump to main code
;================================
  org 4     ;Interrupt routine
    movwf   Wreg            ;Save W
    swapf   STATUS , W      ; nb.  movwf does not affect any STATUS bits
    movwf   StsReg          ;Stave STATUS, without affecting it

    btfss   INTCON , GPIF   ;Test for port B change
    goto    CarryOnInt      ;
    bcf     ContMode
                            ;
    bcf     INTCON , GPIE   ;
    call    Assemble232     ;MUSTN'T Waste ANY TIME UP TO HERE !!!!

    sublw   0x0D            ;Check for [cr] terminating any command,
    btfsc   STATUS, Z       ;  otherwise chars are pushed onto stack
    call    CheckSerCmd     ;Decode the command and do any actions

    bcf     INTCON , GPIF   ;Clear the flag
    bsf     INTCON , GPIE   ;Re-enable the interrupt
;...............
CarryOnInt
    swapf   StsReg , W      ;Restore STATUS
    movwf   STATUS
    swapf   Wreg            ;Restore W
    swapf   Wreg , W        ; nb.  movf affects Z bit

    retfie
;=================================
StartUp 
    	movlw   7
    	movwf   CMCON
    	bsf     STATUS,RP0      ;ram page 1

    	movlw   b'00000001'     ;GP0 is the serial input, all others out
    	movwf   TRISIO          ;
    	movwf   IOC             ;Port change interrupt on GP0

    	movlw   b'00000000'     ;Pull-ups enabled
	movwf	OPTION_REG

    	call    0x3FF           ;Get factory stored OSCCAL value
    	movwf   OSCCAL
    	bcf     STATUS, RP0

	clrf	Flags
	call	ClearStack
	clrf	EEPage

  if RS232POL == 1
    	bcf   SerOut      ;RS232 idle state (note RS232 polarity !)
  else
    	bsf   SerOut
  endif

	call	LongDelay	;Give a bit of time for things to stabilise if all turned on together
	call	LongDelay
	call	LongDelay

	call	IntroMsg
	call	SendFromEE	;

    	movlw   b'10001000'     ;Port Change interrupt
    	movwf   INTCON

MainLoop        ;		;Serial interface is all interrupt driven from now on
	movlw	0
	btfss	PAGELINK	;Link to ground fitted = Page 1
	movlw	0x40
	
	xorwf	EEPage, W	;Test to see if its the same
	btfsc	STATUS, Z
	goto	MainLoop	;and if so do nothing

	call	LongDelay	;Stop any bouncing around

	movlw	0		
	btfss	PAGELINK	
	movlw	0x40
	xorwf	EEPage, W	
	btfsc	STATUS, Z
	goto	MainLoop	
	
	movlw	0		;Otherwise recover the link setting
	btfss	PAGELINK	;and update page address (only used as a flag here)
	movlw	0x40
	movwf	EEPage
	call	SendFromEE
    	goto    MainLoop
;======================================
SendFromEE
	movlw	LOW(ConfigData)
	btfss	PAGELINK
	movlw	LOW(ConfigData2nd)

	movwf	AutoEEADR	
FromEELoop
	movf	AutoEEADR, W
	call	GetEE_12F
	movwf	D3
	incf	AutoEEADR
	movf	AutoEEADR, W
	call	GetEE_12F
	movwf	D2
	incf	AutoEEADR
	movf	AutoEEADR, W
	call	GetEE_12F
	movwf	D1
	incf	AutoEEADR
	movf	AutoEEADR, W
	call	GetEE_12F
	movwf	D0

	call	SendData
	call	SendBack		

	incf	AutoEEADR
	movlw	LOW(EndConfigData)
	btfss	PAGELINK
	movlw	LOW(EndConfigData2nd)

	subwf	AutoEEADR, W		;W = Address - EndOfDataLabel, when 0/+Ve, C=1, exit sending
	btfss	STATUS, C	
	goto	FromEELoop

	return
;------------------------------------
SendData                ;Now send D3/2/1/0  on SPI bus
	movf	D3, W
	movwf	X3
	movf	D2, W
	movwf	X2
	movf	D1, W
	movwf	X1
	movf	D0, W
	movwf	X0

	bcf	LE	
	movlw	d'32'
	movwf	BitCount
SendChipLoop
	bcf	Clk
	rlf	X0
	rlf	X1
	rlf	X2
	rlf	X3	;Working bit in C
	btfsc	STATUS, C	
	bsf	Data		
	btfss	STATUS, C
	bcf	Data

	nop
	nop
	bsf	Clk	;Clocked in on rising edge

	decfsz	BitCount
	goto	SendChipLoop

	bcf	Clk		;Place into rest state
	bcf	Data
	nop
	bsf	LE		;Needs to get back low in 1us
	bcf	LE

	return
;------------------------------------
SendBack              
	movf	D3, W
	movwf	X3
	movf	D2, W
	movwf	X2
	movf	D1, W
	movwf	X1
	movf	D0, W
	movwf	X0

	movlw	d'32'
	movwf	BitCount
SendBackLoop
	rlf	X0
	rlf	X1
	rlf	X2
	rlf	X3

	movlw	"0"
	btfsc	STATUS, C
	movlw	"1"
	call	SendByte	

	movf	BitCount, W
	andlw	0x03
	sublw	0x01
	btfss	STATUS, Z
	goto	NoSp
	movlw	" "
	call	SendByte
NoSp
	decfsz	BitCount
	goto	SendBackLoop

	movlw	" "
	call	SendByte
	movlw	"0"
	call	SendByte
	movlw	"x"
	call	SendByte

	swapf	D3, W
	andlw	0x0F
	call	SendHex
	movf	D3, W
	andlw	0x0F
	call	SendHex

	movlw	" "
	call	SendByte
	
	swapf	D2, W
	andlw	0x0F
	call	SendHex
	movf	D2, W
	andlw	0x0F
	call	SendHex

	movlw	" "
	call	SendByte

	swapf	D1, W
	andlw	0x0F
	call	SendHex
	movf	D1, W
	andlw	0x0F
	call	SendHex

	movlw	" "
	call	SendByte
	
	swapf	D0, W
	andlw	0x0F
	call	SendHex
	movf	D0, W
	andlw	0x0F
	call	SendHex
	
	call	CrLf

	return
;------------------------------------
DecodeHex           ;Single ASCII character to binary 
    movwf   Temp            ;Uses Temp
    movlw   0x30
    subwf   Temp            ;subtract $30 and leave in Temp
    btfss   STATUS , C      ;if < $30, result -ve so C=0, error, set=0
    clrf    Temp

    movf    Temp , W        
    sublw   d'9'            ;W = 9 - W
    btfsc   STATUS , C      ;if -ve, C=0, char > 9, so subtract 7
    goto    ConvDone
    movlw   d'7'
    subwf   Temp            ;
ConvDone
    movf    Temp , W        ;
    andlw   0x0F            ;mask out any rubbish left in
    return              
;------------------------------------
SendHex         ;Send a single Nibble as RS232
    andlw   0x0F
    addlw   0x30
    movwf   Temp
    sublw   0x39
    btfsc   STATUS , C
    goto    CharDone
    movlw   7
    addwf   Temp
CharDone
    movf    Temp , W
    call    SendByte
    return
;------------------------------
CrLf
    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte
    return
;---------------------------------
SendByte                    
	movwf   Temp
 if RS232POL == 1
    	bsf     SerOut
 else
    	bcf     SerOut             ;start bit
 endif
	call    HalfBitDelay        ;
	call    HalfBitDelay
	movlw   8
	movwf   Counter
SendDatLoop
	rrf     Temp                ;bit into C

 if RS232POL == 1
	btfsc   STATUS , C
    	bcf     SerOut
	btfss   STATUS , C
    	bsf     SerOut             ;could have 2us jitter depending on 1/0
 else
	btfsc   STATUS , C
    	bsf     SerOut
	btfss   STATUS , C
    	bcf     SerOut             ;could have 2us jitter depending on 1/0
 endif

	call    HalfBitDelay
	call    HalfBitDelay
	decfsz  Counter
	goto    SendDatLoop         ;loop 8 + BitDelay long

 if RS232POL == 1
    	bcf     SerOut
 else
    	bsf     SerOut
 endif
	call    HalfBitDelay    
	call    HalfBitDelay        ;Two stop bits
	call    HalfBitDelay
	call    HalfBitDelay

	return
;-------------------------------
CheckSerCmd             ;[cr] terminator has been received, so decode
                        ; the commands in here. The [cr]  Will be in Stack1
	clrf	D0
	clrf	D1
	clrf	D2
	clrf	D3

   	movlw   "R"         ;Rxxxx  R register data data
    	subwf   StackA, W   ;Stack A/9/8/7/6/5/4/3/2/1
    	btfsc   STATUS, Z   ;      R x x x x x x x x [cr]
    	goto    RCmd32  

	movlw   "R"         ;
    	subwf   Stack8, W   ;Stack 8/7/6/5/4/3/2/1
    	btfsc   STATUS, Z   ;      R x x x x x x [cr]
    	goto    RCmd24  

   	movlw   "R"         ;
    	subwf   Stack6, W   ;Stack 6/5/4/3/2/1
    	btfsc   STATUS, Z   ;      R x x x x [cr]
    	goto    RCmd16  

  	movlw   "R"         ;
    	subwf   Stack4, W   ;Stack 4/3/2/1
    	btfsc   STATUS, Z   ;      R x x [cr]
    	goto    RCmd8

 	movlw   "T"         ;
    	subwf   Stack5, W   ;Stack 5/4/3/2/1
    	btfsc   STATUS, Z   ;      T E S T [cr]
    	goto    TCmd  

	movlw   "I"         ;
    	subwf   Stack5, W   ;Stack 5/4/3/2/1
    	btfsc   STATUS, Z   ;      I N I T [cr]
    	goto    ICmd  

 	movlw   "W"         ;
    	subwf   Stack2, W   ;Stack 5/4/3/2/1
    	btfsc   STATUS, Z   ;            W [cr]
    	goto    WCmd  

    	goto    DoneCmd
;.............
RCmd32		;Extract variable length hex data and left align into D3/0, depending on value of bytes entered
	movf	Stack9, W
	call	DecodeHex
	movwf	D3
	swapf	D3
	movf	Stack8, W
	call	DecodeHex
	addwf	D3
RCmd24
	movf	Stack7, W
	call	DecodeHex
	movwf	D2
	swapf	D2
	movf	Stack6, W
	call	DecodeHex
	addwf	D2
RCmd16
	movf	Stack5, W
	call	DecodeHex
	movwf	D1
	swapf	D1
	movf	Stack4, W
	call	DecodeHex
	addwf	D1
RCmd8
	movf	Stack3, W
	call	DecodeHex
	movwf	D0
	swapf	D0
	movf	Stack2, W
	call	DecodeHex
	addwf	D0


	call	SendBack
	call	SendData  

	movf	D3, W		;Make a backup in X3-0 in case TEST command is needed
	movwf	X3
	movf	D2, W
	movwf	X2
	movf	D1, W
	movwf	X1
	movf	D0, W
	movwf	X0
	goto    DoneCmd

;.............
ICmd			;INIT,  sends data from EE
	movlw	"N"
	subwf	Stack4
	btfss	STATUS, Z
	goto	DoneCmd		;Get out if INIT has not been entered in full 

	movlw	"I"
	subwf	Stack3
	btfss	STATUS, Z
	goto	DoneCmd	

	movlw	"T"
	subwf	Stack2
	btfss	STATUS, Z
	goto	DoneCmd	


	call	CrLf
	movlw	"I"
	call	SendByte
	movlw	"n"
	call	SendByte
	movlw	"i"
	call	SendByte
	movlw	"t"
	call	SendByte
	call	CrLf

	call	SendFromEE
	goto	DoneCmd
;..............
WCmd			;W stores last data to appropriate location in EE
	movf	X0, W
	andlw	0x0F		;Extract Address.  Data is stored in order sent, so
	movwf	Temp		;   need to search EE for the right location

	movlw	LOW(ConfigData + 3)	;Only interested in LS Byte in each case
	btfss	PAGELINK
	movlw	LOW(ConfigData2nd + 3)



	movwf	AutoEEADR	
SearchLoop
	movf	AutoEEADR, W
	call	GetEE_12F	;Temp is not used in this call, so can assume to be preserved
	andlw	0x0F
	subwf	Temp, W
	btfsc	STATUS, Z
	goto	FoundIt

	movlw	4
	addwf	AutoEEADR

	movlw	LOW(EndConfigData)
	btfss	PAGELINK
	movlw	LOW(EndConfigData2nd)

	subwf	AutoEEADR, W		;W = Address - EndOfDataLabel, when 0/+Ve, C=1, exit sending
	btfss	STATUS, C	
	goto	SearchLoop

	movlw	"?"
	call	SendByte
	movlw	"?"
	call	SendByte
	call	CrLf

	goto	DoneCmd		;Fall out without doing anything if the right location doesn't exist
;..........			; Data cannot be stored from scratch, only into already existing addresses
FoundIt
	movf	X0, W		;Use backup data from X, as D was cleared in the main loop
	call	StoreEE_12F        ;Enter with W = EE data,  AutoEEADR = EE address
	decf	AutoEEADR
	movf	X1, W
	call	StoreEE_12F 
	decf	AutoEEADR
	movf	X2, W
	call	StoreEE_12F 
	decf	AutoEEADR
	movf	X3, W
	call	StoreEE_12F 

	movlw	" "
	call	SendByte
	movlw	"0"
	call	SendByte
	movlw	"x"
	call	SendByte

	swapf	X3, W
	andlw	0x0F
	call	SendHex
	movf	X3, W
	andlw	0x0F
	call	SendHex

	movlw	" "
	call	SendByte
	
	swapf	X2, W
	andlw	0x0F
	call	SendHex
	movf	X2, W
	andlw	0x0F
	call	SendHex

	movlw	" "
	call	SendByte

	swapf	X1, W
	andlw	0x0F
	call	SendHex
	movf	X1, W
	andlw	0x0F
	call	SendHex

	movlw	" "
	call	SendByte
	
	swapf	X0, W
	andlw	0x0F
	call	SendHex
	movf	X0, W
	andlw	0x0F
	call	SendHex

	movlw	" "
	call	SendByte
	movlw	"W"
	call	SendByte
	movlw	"r"
	call	SendByte
	movlw	"i"
	call	SendByte
	movlw	"t"
	call	SendByte
	movlw	"t"
	call	SendByte
	movlw	"e"
	call	SendByte
	movlw	"n"
	call	SendByte


	call	CrLf

	goto	DoneCmd
;..............

TCmd			;Test mode, sends data continuously to chip for testing SPI interface ONLY
	movlw	"E"
	subwf	Stack4
	btfss	STATUS, Z
	goto	DoneCmd		;Get out if TEST has not been entered in full 

	movlw	"S"
	subwf	Stack3
	btfss	STATUS, Z
	goto	DoneCmd	

	movlw	"T"
	subwf	Stack2
	btfss	STATUS, Z
	goto	DoneCmd	

	bsf	ContMode

	movf	X3, W		;Recover original D data coz it was deleted during the TEST command
	movwf	D3
	movf	X2, W
	movwf	D2
	movf	X1, W
	movwf	D1
	movf	X0, W
	movwf	D0


	call	SendBack
	call	SendBack
	call	SendBack
	
	movlw	"R"
	call	SendByte
	movlw	"e"
	call	SendByte
	movlw	"p"
	call	SendByte
	movlw	"e"
	call	SendByte
	movlw	"a"
	call	SendByte
	movlw	"t"
	call	SendByte
	movlw	"i"
	call	SendByte
	movlw	"n"
	call	SendByte
	movlw	"g"
	call	SendByte
	call	CrLf

TestModeLoop
	call	SendData
	btfsc	ContMode	;Cleared inside interrupt, by any key entry

	goto	TestModeLoop
;.............
DoneCmd
	call	ClearStack      ;Prevents spurious commands
	call	CrLf
    return
;------------------------------- 
Assemble232             ;Get here after Port B interrupt
    call    HalfBitDelay

 if RS232POL == 1
    btfss   SerIn     
 else
    btfsc   SerIn
 endif

    goto    GetOut232   ;Invalid start bit

    movlw   d'8'        ;27 clocks after edge to here
    movwf   Counter
    clrf    RxChar
Rx232Loop
    call    HalfBitDelay
    call    HalfBitDelay
    bcf     STATUS , C

 if RS232POL == 1
    btfss   SerIn
 else
    btfsc   SerIn
 endif

    bsf     STATUS , C
    rrf     RxChar            
    decfsz  Counter
    goto    Rx232Loop   ;this loop 2 * HalfBitDelay + 9
    call    HalfBitDelay ;Part into stop bit - should be no more transitions


    movlw   "a"             
    subwf   RxChar, W       ;W = Char - "a"
    btfss   STATUS, C       ;if-ve (C=0) char < "a" so ignore
    goto    UpCaseDone

    movf    RxChar, W
    sublw   "z"             ;W = "z" - Char
    btfss   STATUS, C       ;if -ve (C=0)  char > "z" so ignore
    goto    UpCaseDone

    bcf     RxChar, 5       ;Reset bit 5 to force upper case
UpCaseDone
    movf    RxChar, W
    call    PushStack       ;RxChar >> Stack1, previous 5 chars move down one
    movf    RxChar , w      ;Returns with data in W as well as on stack
    
GetOut232
    return                  ;received byte in RxChar and W
;-------------------------
HalfBitDelay            ;1200 baud needs 833us, so delay 414 clocks here
    movlw   BAUDCONST   ; for a total of 414 * 2 + 8 = 833 clocks
	movwf   DelCount
BaudDelLoop             ;Total delay = 3.N + 5 = 414
	decfsz  DelCount
	goto    BaudDelLoop
	return
;-------------------------------
LongDelay
	movlw   d'150'      ;Approx 300ms delay
	movwf   Temp2
LongDelLoop1
	call	Delay
	call	Delay
	decfsz	Temp2
	goto	LongDelLoop1
	return
;------------------------------------
Delay                       ;1ms delay
    movlw   d'199'  
    movwf   DelCount
DelLoop
    nop
    nop
    decfsz  DelCount
    goto    DelLoop
    return
;------------------------------------
BCD2Bin		;Simple two digit one byte version for values up to 39
	movwf	Temp
	
	swapf	Temp, W
	andlw	0x0F		;Recover tens digit
	movwf	Temp2
	rlf	Temp2
	rlf	Temp2, W	
	andlw	0x3C
	movwf	Temp2		;Temp2 = Tens digit * 4
	swapf	Temp, W
	andlw	0x0F		;Recover tens digit
	addwf	Temp2		;Temp2 = Tens digit * 5
	bcf	STATUS, C
	rlf	Temp2		;Temp2 = Tens digit * 10

	movf	Temp, W
	andlw	0x0F		;Recover digits
	addwf	Temp2, W	;W = binary versino of BCD intry

	return
;------------------------------------
GetEE_12F                   ;Enter with address in W
    bsf     STATUS, RP0
    movwf   EEADR
    bsf     EECON1, RD
    movf    EEDATA, W
    bcf     STATUS, RP0
    return                  ;Exit with data in W
;-----------------------------------------
;-------------------------------------------
StoreEE_12F        ;Enter with W = EE data,  AutoEEADR = EE address
    bsf     STATUS, RP0      ;All EE addresses in bank 1
    movwf   EEDATA
    bcf     STATUS, RP0
    movf    AutoEEADR, W
    bsf     STATUS, RP0
    movwf   EEADR    

    bsf     EECON1 , WREN    ;Enable EEprom writing
    movlw   0x55
    movwf   EECON2
    movlw   0xAA
    movwf   EECON2
    bsf     EECON1 , WR

    bcf     STATUS , RP0
EEwaitW
    btfss   PIR1 , EEIF
    goto    EEwaitW

    bcf     PIR1 , EEIF      ;has to be cleared manually
    return
;-------------------------
PushStack
    movf    Stack9 , W
    movwf   StackA          
    movf    Stack8 , W
    movwf   Stack9
    movf    Stack7 , W
    movwf   Stack8              ;StackA contains oldest data
    movf    Stack6 , W
    movwf   Stack7
    movf    Stack5 , W
    movwf   Stack6          
    movf    Stack4 , W
    movwf   Stack5
    movf    Stack3 , W
    movwf   Stack4
    movf    Stack2 , W
    movwf   Stack3
    movf    Stack1 , W
    movwf   Stack2
    movf    RxChar , W
    movwf   Stack1
    return
;-------------------------
ClearStack
    movlw   0
    movwf   StackA
    movwf   Stack9
    movwf   Stack8
    movwf   Stack7
    movwf   Stack6
    movwf   Stack5
    movwf   Stack4
    movwf   Stack3
    movwf   Stack2
    movwf   Stack1
    return
;------------------------------------

IntroMsg
	movlw	"L"
	call	SendByte
	movlw	"M"
	call	SendByte
	movlw	"X"
	call	SendByte
	movlw	"2"
	call	SendByte
	movlw	"5"
	call	SendByte
	movlw	"4"
	call	SendByte
	movlw	"1"
	call	SendByte
	movlw	" "
	call	SendByte
	movlw	"C"
	call	SendByte
	movlw	"T"
	call	SendByte
	movlw	"R"
	call	SendByte
	movlw	"L"
	call	SendByte
	call	CrLf

	movlw	"R"
	call	SendByte
	movlw	"x"
	call	SendByte
	movlw	"x"
	call	SendByte
	movlw	"x"
	call	SendByte
	movlw	"x"
	call	SendByte
	movlw	"x"
	call	SendByte
	movlw	"x"
	call	SendByte
	movlw	"x"
	call	SendByte
	movlw	"x"
	call	CrLf

	movlw	"W"
	call	SendByte
	call	CrLf

	movlw	"I"
	call	SendByte
	movlw	"N"
	call	SendByte
	movlw	"I"
	call	SendByte
	movlw	"T"
	call	SendByte
	call	CrLf

	movlw	"T"
	call	SendByte
	movlw	"E"
	call	SendByte
	movlw	"S"
	call	SendByte
	movlw	"T"
	call	SendByte
	call	CrLf

	call	CrLf
	return		
;---------------------------------------
    cblock 0x20         ;64 Bytes available
        Wreg, StsReg
        Temp, Temp2
	Flags
	SaveBit
        BitCount
        Counter
        DelCount
	EEPage
        AutoEEADR
        D0, D1, D2, D3
	X0, X1, X2, X3

        RxChar
        Stack1, Stack2, Stack3, Stack4, Stack5, Stack6, Stack7, Stack8, Stack9, StackA
    endc
;----------------------
	end

